#    Copyright 2014 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

import yaml


def _get_config_path():
    if os.path.exists('/etc/network_checker/config.yaml'):
        return '/etc/network_checker/config.yaml'
    return os.path.join(
        os.path.dirname(os.path.realpath(__file__)),
        'config.yaml')


def get_config():
    with open(_get_config_path()) as conf:
        return yaml.load(conf.read())
